import * as THREE from "three";

export const autoScale = (geometry: THREE.BufferGeometry) => {
  // 计算模型的边界
  geometry.computeBoundingBox();
  // 获取模型的大小
  const modelSize = geometry.boundingBox!.getSize(new THREE.Vector3());
  const maxSize = Math.max(modelSize.x, modelSize.y, modelSize.z);
  // 根据模型的大小设置一个缩放因子
  const scaleFactor = 2 / maxSize; // 假设我们希望模型最大为2单位
  return scaleFactor;
};

export const getTriangleVertexs = (
  geometry: THREE.BufferGeometry
): THREE.Vector3[][] => {
  const positions = geometry.attributes.position;
  const triangles: THREE.Vector3[][] = [];

  const getTriangleVertices = (
    index1: number,
    index2: number,
    index3: number
  ) => [
    new THREE.Vector3(
      positions.getX(index1),
      positions.getY(index1),
      positions.getZ(index1)
    ),
    new THREE.Vector3(
      positions.getX(index2),
      positions.getY(index2),
      positions.getZ(index2)
    ),
    new THREE.Vector3(
      positions.getX(index3),
      positions.getY(index3),
      positions.getZ(index3)
    ),
  ];

  if (geometry.index) {
    const indices = geometry.index.array;
    for (let i = 0; i < indices.length; i += 3) {
      triangles.push(
        getTriangleVertices(indices[i], indices[i + 1], indices[i + 2])
      );
    }
  } else {
    for (let i = 0; i < positions.count; i += 3) {
      triangles.push(getTriangleVertices(i, i + 1, i + 2));
    }
  }

  return triangles;
};

export const getOffsetTriangles = (
  triangles: THREE.Vector3[][],
  offset: number
): THREE.Vector3[][] => {
  const newTriangles: THREE.Vector3[][] = [];

  triangles.forEach((triangle) => {
    // 计算三角形的法线
    const normal = new THREE.Vector3();
    const edge1 = new THREE.Vector3().subVectors(triangle[1], triangle[0]);
    const edge2 = new THREE.Vector3().subVectors(triangle[2], triangle[0]);
    normal.crossVectors(edge1, edge2).normalize();

    // 使用法线和偏移量计算新的三角形顶点
    const offsetNormal = normal.clone().multiplyScalar(offset);
    const newTriangle = triangle.map((vertex) =>
      vertex.clone().sub(offsetNormal)
    );
    newTriangles.push(newTriangle);
  });

  return newTriangles;
};

export const createEnclosedGeometry = (
  triangles: THREE.Vector3[][],
  newTriangles: THREE.Vector3[][]
): THREE.BufferGeometry => {
  const geometry = new THREE.BufferGeometry();
  const vertices: THREE.Vector3[] = [];
  const indices: number[] = [];

  const processedEdges: Set<string> = new Set();

  function edgeId(v1: THREE.Vector3, v2: THREE.Vector3): string {
    return `${v1.x},${v1.y},${v1.z}->${v2.x},${v2.y},${v2.z}`;
  }

  triangles.forEach((triangle, index) => {
    const innerTriangle = newTriangles[index];

    triangle.forEach((vertex) => {
      vertices.push(vertex);
    });

    innerTriangle.forEach((vertex) => {
      vertices.push(vertex);
    });

    // Process the three edges for each triangle
    for (let i = 0; i < 3; i++) {
      const next = (i + 1) % 3;
      const edge = edgeId(triangle[i], triangle[next]);

      if (!processedEdges.has(edge)) {
        processedEdges.add(edge);

        // Add the vertices for the quad (two triangles)
        const quadVertices = [
          triangle[i],
          triangle[next],
          innerTriangle[i],
          innerTriangle[next],
        ];

        quadVertices.forEach((vertex) => {
          vertices.push(vertex);
        });

        const baseIndex = vertices.length - 4;

        // First triangle of the quad
        indices.push(baseIndex, baseIndex + 2, baseIndex + 1);
        // Second triangle of the quad
        indices.push(baseIndex + 1, baseIndex + 2, baseIndex + 3);
      }
    }
  });

  geometry.setFromPoints(vertices);
  geometry.setIndex(indices);

  return geometry;
};

// export const createEnclosedGeometry = (
//   triangles: THREE.Vector3[][],
//   newTriangles: THREE.Vector3[][]
// ): THREE.BufferGeometry => {
//   const geometry = new THREE.BufferGeometry();

//   const vertices: number[] = [];
//   const indices: number[] = [];

//   // 添加原始三角形和新三角形的顶点到vertices数组
//   triangles.forEach((triangle) => {
//     triangle.forEach((v) => {
//       vertices.push(v.x, v.y, v.z);
//     });
//   });

//   newTriangles.forEach((triangle) => {
//     triangle.forEach((v) => {
//       vertices.push(v.x, v.y, v.z);
//     });
//   });

//   const totalTriangles = triangles.length;

//   // 创建索引来连接triangles和newTriangles中的三角形
//   for (let i = 0; i < totalTriangles; i++) {
//     const offset = totalTriangles * 3; // newTriangles在vertices中的偏移量

//     const a1 = i * 3;
//     const b1 = a1 + 1;
//     const c1 = a1 + 2;

//     const a2 = offset + i * 3;
//     const b2 = a2 + 1;
//     const c2 = a2 + 2;

//     // 添加原始和新三角形的索引
//     indices.push(a1, b1, c1);
//     indices.push(a2, c2, b2);

//     // 为每一对相邻的三角形添加边的索引
//     indices.push(a1, b1, a2);
//     indices.push(b1, b2, a2);

//     indices.push(b1, c1, b2);
//     indices.push(c1, c2, b2);

//     indices.push(c1, a1, c2);
//     indices.push(a1, a2, c2);
//   }

//   geometry.setIndex(indices);
//   geometry.setAttribute(
//     "position",
//     new THREE.Float32BufferAttribute(vertices, 3)
//   );
//   geometry.computeVertexNormals(); // 计算顶点法线

//   return geometry;
// };
